#!/bin/bash

# Script positional parameters:
#	1. INPUT_FASTA Input sequence(s) for one organism.
#	2. OUTDIR Name for output sub-directory. It will be created if it doesn't
#        already exist otherwise it will be over-written.

# Final prediction is
#	OUTDIR/tax.final

INPUT_FASTA=$1
OUTDIR=$2

if [ x$INPUT_FASTA == x ] ; then
	echo "Usage: $0 [optional_path]input.fasta outputdir"
	echo "Consensus prediction is written to outputdir/tax.final"
	echo ""
	exit 0
fi

REFDIR=$SERRATAX_REFDIR
LIBDIR=$SERRATAX_LIBDIR

if [ x$REFDIR == x ] ; then
	echo '$REFDIR not set' >> /dev/stderr
	exit 1
fi

if [ x$OUTDIR == x ] ;then
	echo Missing OUTDIR >> /dev/stderr
	exit 1
fi

if [ ! -f $INPUT_FASTA ] ; then
	echo Not found INPUT_FASTA=$INPUT_FASTA >> /dev/stderr
	exit 1
fi

if [ ! -d $REFDIR ] ; then
	echo Directory not found REFDIR=$REFDIR >> /dev/stderr
	exit 1
fi

if [ -d $OUTDIR ] ; then
	rm -f $OUTDIR/*
	ex=0 # ignore failure
else
	mkdir -p $OUTDIR
	ex=$?
fi

if [ $ex != 0 ] ; then
	echo exit $ex rm/mkdir $OUTDIR >> /dev/stderr
	exit 1
fi

startsecs=`date "+%s"`
REPORT=$OUTDIR/report.txt

getorf -sequence $INPUT_FASTA \
  -minsize 300 \
  -outseq $OUTDIR/orfs.fa
ex=$?
if [ $ex != 0 ] ; then
	echo exit $ex getorf >> /dev/stderr
	echo exit $ex getorf >> $REPORT
	exit 1
fi

n=`cat $OUTDIR/orfs.fa | grep -c "^>"`
echo "$n ORFs" >> $REPORT

usearch -fastx_relabel $OUTDIR/orfs.fa \
  -prefix QORF. \
  -fastaout $OUTDIR/orfs2.fa
ex=$?
if [ $ex != 0 ] ; then
	echo exit $ex fastx_relabel >> /dev/stderr
	echo exit $ex fastx_relabel >> $REPORT
	exit 1
fi

usearch -ublast $OUTDIR/orfs2.fa \
  -db $REFDIR/doms.fa \
  -evalue 1e-6 \
  -userout $OUTDIR/ublast_orfs_domains.tsv \
  -userfields query+target+qlo+qhi+ql+qcov+tlo+thi+tl+tcov+id+evalue
ex=$?
if [ $ex != 0 ] ; then
	echo exit $ex ublast >> /dev/stderr
	echo exit $ex ublast >> $REPORT
	exit 1
fi

n=`cat $OUTDIR/ublast_orfs_domains.tsv | wc -l`
echo "$n ublast alignments" >> $REPORT

$LIBDIR/serratus_covann_select.py $OUTDIR $REPORT
ex=$?
if [ $ex != 0 ] ; then
	echo exit $ex serratus_covann_select >> /dev/stderr
	echo exit $ex serratus_covann_select >> $REPORT
	exit 1
fi

n=`cat $OUTDIR/doms.txt | wc -l`
if [ $n == 0 ] ; then
	echo "No domains found" >> $REPORT
	exit 1
fi

for x in `cat $OUTDIR/doms.txt`
do 
	muscle -in $OUTDIR/$x.fa -out $OUTDIR/$x.afa -maxiters 2
	ex=$?
	if [ $ex != 0 ] ; then
		echo exit $ex muscle MSA >> /dev/stderr
		echo exit $ex muscle MSA >> $REPORT
		exit 1
	fi

	muscle -maketree -in $OUTDIR/$x.afa -out $OUTDIR/$x.nj.newick
	ex=$?
	if [ $ex != 0 ] ; then
		echo exit $ex muscle NJ >> /dev/stderr
		echo exit $ex muscle NJ >> $REPORT
		exit 1
	fi

	usearch -tree_cvt $OUTDIR/$x.nj.newick \
	  -tabbedout $OUTDIR/$x.nj.tsv
	ex=$?
	if [ $ex != 0 ] ; then
		echo exit $ex tree_cvt >> /dev/stderr
		echo exit $ex tree_cvt >> $REPORT
		exit 1
	fi

	$LIBDIR/serratus_classify.py $OUTDIR/$x.nj.tsv \
	  $REPORT \
	  > $OUTDIR/$x.nj.classify
	ex=$?
	if [ $ex != 0 ] ; then
		echo exit $ex serratus_classify $OUTDIR/$x.nj.tsv >> /dev/stderr
		echo exit $ex serratus_classify $OUTDIR/$x.nj.tsv >> $REPORT
		exit 1
	fi
done

endsecs=`date "+%s"`
elapsedsecs=$((endsecs-startsecs))
echo "$elapsedsecs secs" >> $REPORT

$LIBDIR/serratus_classify_consensus.py $REPORT \
  $OUTDIR/*.classify \
  > $OUTDIR/tax.final
if [ $ex != 0 ] ; then
	echo exit $ex serratus_classify_consensus $OUTDIR/$x.nj.tsv >> /dev/stderr
	echo exit $ex serratus_classify_consensus $OUTDIR/$x.nj.tsv >> $REPORT
	exit 1
fi
