# My reformatted NCBI taxonomy tree format
# Tabbed, fields:
#	0. node 1. parent 2. name 3. rank

import sys
import progress
import die

DefaultFileName = "/e/res/ncbi/out/taxtree.txt"

Nodes = []
NodeToParent = {}
NodeToName = {}
NodeToRank = {}
ParentToChildren = {}

def Load(FileName = DefaultFileName):
	f = open(FileName)
	progress.FileInit(f, "taxtree")
	while 1:
		Line = f.readline()
		progress.FileStep()
		if len(Line) == 0:
			progress.FileDone()
			print >> sys.stderr, "%u nodes" % len(Nodes)
			break
	
		Fields = Line[:-1].split('\t')
		if len(Fields) != 4:
			print >> sys.stderr, "Expected 4 fields got %d in %s" % (len(Fields), FileName)
			assert False

		Node = int(Fields[0])
		Parent = int(Fields[1])
		Name = Fields[2]
		Rank = Fields[3]

		Nodes.append(Node)

		if Parent < 0 or Node == Parent:
			NodeToParent[Node] = None
		else:
			NodeToParent[Node] = Parent
		NodeToName[Node] = Name
		NodeToRank[Node] = Rank
	
		try:
			ParentToChildren[Parent].append(Node)
		except:
			ParentToChildren[Parent] = [ Node ]

def GetName(Node, Default = ""):
	try:
		Name = NodeToName[Node]
	except:
		Name = Default
	if Name == "":
		die.Die("GetName(%u) not found" % Node)
	return Name

def GetRank(Node, Default = ""):
	try:
		Rank = NodeToRank[Node]
	except:
		Rank = Default
	if Rank == "":
		die.Die("GetRank(%d), not found" % Node)
	return Rank

def GetPathToRoot(Node):
	Path = [ Node ]
	while 1:
		if Node == 1:
			return Path
		try:
			Node = NodeToParent[Node]
		except:
			return Path
		Path.append(Node)

def GetSpeciesNode(Node):
	while 1:
		if Node == 1:
			return -1
		try:
			Rank = NodeToRank[Node]
		except:
			return -1
		if Rank == "species":
			return Node
		Node = NodeToParent[Node]

def GetSpeciesName(Node, Default = ""):
	Node = GetSpeciesNode(Node)
	if Node == -1:
		if Default == "":
			die.Die("Species not found for taxid=%u" % Node)
		return Default
	return NodeToName[Node]

def GetGenusNode(Node):
	while 1:
		if Node == 1:
			return -1
		Rank = NodeToRank[Node]
		if Rank == "genus":
			return Node
		Node = NodeToParent[Node]

def GetGenusName(Node, Default = ""):
	Node = GetGenusNode(Node)
	if Node == -1:
		if Default == "":
			die.Die("Genus not found for taxid=%u" % Node)
		return Default
	return NodeToName[Node]

def GetFamilyNode(Node):
	while 1:
		if Node == 1:
			return -1
		try:
			Rank = NodeToRank[Node]
		except:
			return -1
		if Rank == "family":
			return Node
		Node = NodeToParent[Node]

def GetFamilyName(Node, Default = ""):
	Node = GetFamilyNode(Node)
	if Node == -1:
		if Default == "":
			die.Die("Family not found for taxid=%u" % Node)
		return Default
	return NodeToName[Node]

def GetPhylumNode(Node):
	while 1:
		if Node == 1:
			return -1
		Rank = NodeToRank[Node]
		if Rank == "phylum":
			return Node
		Node = NodeToParent[Node]

def GetPhylumName(Node, Default = ""):
	Node = GetPhylumNode(Node)
	if Node == -1:
		if Default == "":
			die.Die("Phylum not found for taxid=%u" % Node)
		return Default
	return NodeToName[Node]

def GetAncNode(Node, Rank):
	while 1:
		if Node == 1:
			return -1
		try:
			Rank2 = NodeToRank[Node]
		except:
			return -1
		if Rank2 == Rank:
			return Node
		Node = NodeToParent[Node]

def GetAncName(Node, Rank, Default = ""):
	AncNode = GetAncNode(Node, Rank)
	if AncNode == -1:
		if Default == "":
			die.Die("GetAncNode(%u, %s), not found" % (Node, Rank))
		return Default
	return GetName(AncNode)

def GetDomainNode(Node):
	while 1:
		if Node == 1:
			return -1
		try:
			Rank = NodeToRank[Node]
		except:
			return -1
		if Rank == "kingdom" or Rank == "superkingdom":
			return Node
		Node = NodeToParent[Node]

def GetDomainName(Node, Default = ""):
	Node = GetDomainNode(Node)
	if Node == -1:
		if Default == "":
			die.Die("Domain not found for taxid=%u" % Node)
		return Default
	try:
		Name = NodeToName[Node]
	except:
		Name = "?"

	return Name

def GetKingdomNode(Node, Default = ""):
	return GetDomainNode(Node, Default)

def GetKingdomName(Node, Default = ""):
	return GetDomainName(Node, Default)

def IsAncNode(Node, AncNode):
	while 1:
		if Node == 1:
			return False
		try:
			Node = NodeToParent[Node]
		except:
			return False
		if Node == AncNode:
			return True

def GetProkLCANode(TaxId1, TaxId2):
	Path1 = GetProkPathToRoot(TaxId1)
	Path2 = GetProkPathToRoot(TaxId2)
	for i in range(0, 7):
		Node1 = Path1[6-i]
		Node2 = Path2[6-i]
		if Node1 == Node2:
			return Node1
	return 1

def GetLCA(TaxIds):
	n = len(TaxIds)
	if n == 0:
		return 1

	LCAPath = GetPathToRoot(TaxIds[0])

	for i in range(1, n):
		Path = GetPathToRoot(TaxIds[i])
		NewLCAPath = []
		for k in range(0, len(Path)):
			Node = Path[k]
			if Node in LCAPath:
				NewLCAPath.append(Node)
		assert len(NewLCAPath) > 0
		LCAPath = NewLCAPath
	return LCAPath[0]

def GetProkPathToRoot(Node):
	#                      0         1        2        3         4        5         6 = 7
	Ranks = [ "superkingdom", "phylum", "class", "order", "family", "genus", "species" ]
	Path  = [             -1,       -1,      -1,      -1,       -1,      -1,       -1  ]
	n = 0
	while 1:
		if Node == 1:
			return Path
		Rank = NodeToRank[Node]
		try:
			Ix = Ranks.index(Rank)
		except:
			Ix = -1
		if Ix >= 0:
			Path[Ix] = Node
			n += 1
		Node = NodeToParent[Node]
	return Path

def GetProkPathToRootNames(Node):
	Path = GetProkPathToRoot(Node)
	s = ""
	for Node in Path:
		if Node == -1:
			if len(s) == 0:
				s = "_ERROR_;"
			break
		Name = GetName(Node)
		s += Name + ";"
	return s

def GetUtaxStr(TaxId):
	Path = GetPathToRoot(TaxId)
	s = ""
	Path = Path[::-1]
	for Node in Path:
		Name = GetName(Node, "?")
		Rank = GetRank(Node, "?")
		if Rank == "species":
			c = 's'
		elif Rank == "genus":
			c = 'g'
		elif Rank == "family":
			c = 'f'
		elif Rank == "class":
			c = 'c'
		elif Rank == "order":
			c = 'o'
		elif Rank == "superkingdom":
			c = 'k'
		else:
			continue
		if s != "":
			s += ","
		Name = Name.replace(" ", "_")
		s += c
		s += ":"
		s += Name
	return s

def GetTaxStr(TaxId):
	Path = GetPathToRoot(TaxId)

	Names = []
	Ranks = []
	N = len(Path)
	for i in range(0, N):
		Node = Path[N-i-1]
		Name = GetName(Node, "?")
		Rank = GetRank(Node, "?")
		Names.append(Name)
		Ranks.append(Rank)

	s = ""
	for i in range(1, N):
		if i > 1:
			s += ","
		Name = Names[i]
		Rank = Ranks[i]
		if Rank == "no_rank" and i > 0:
			PrevRank = Ranks[i-1]
			Rank = "sub." + PrevRank
		s += Rank + ":" + Name
	return s

def IsLeaf(Node):
	try:
		Children = ParentToChildren[Node]
	except:
		return True
	return Children == None

def GetFirstChild(Node):
	try:
		Children = ParentToChildren[Node]
		return Children[0]
	except:
		return None

def GetNextSibling(Node):
	Parent = NodeToParent[Node]
	try:
		Children = ParentToChildren[Parent]
	except:
		return None
	n = len(Children)
	for i in range(0, n):
		if Children[i] == Node:
			if i + 1 < n:
				return Children[i+1]
			else:
				return None
	assert False

def IsRoot(Node):
	Parent = NodeToParent[Node]
	return Parent == None

def GetFirstDepthFirst():
	N = len(Nodes)
	Node = Nodes[0]
	n = 0
	while True:
#		print(Node, IsLeaf(Node))
		if IsLeaf(Node):
			return Node
		Child = GetFirstChild(Node)
#		print("GetFirstChild(%d) = %d" % (Node, Child))
		Node = Child
		n += 1
		assert n < N

def GetNextDepthFirst(Node):
	if Node == None:
		return GetFirstDepthFirst()

	if IsRoot(Node):
		return None

	Node2 = GetNextSibling(Node)
	if Node2 == None:
		return NodeToParent[Node]
	N = len(Nodes)

	n = 0
	Node = Node2
	while True:
		if IsLeaf(Node):
			return Node
		Node = GetFirstChild(Node)
		n += 1
		assert n < N

def DepthFirst(OnNode):
	Node = GetFirstDepthFirst()
	OnNode(Node)
	while True:
		Node = GetNextDepthFirst(Node)
		if Node == None:
			return
		OnNode(Node)
